(function ($, Drupal, once, window) {
  Drupal.behaviors.vlsuite_layout_builder = {
    attach(context) {
      once(
        'vlsuite-layout-builder-live-preview',
        '.layout-builder__live-preview__toggler__link',
        context,
      ).forEach(layoutBuilderLivePreview);
      once(
        'vlsuite-layout-builder-edit-live-preview',
        'form[data-layout-builder-target-highlight-id] [data-editor-for]',
        context,
      ).forEach(layoutBuilderEditLivePreview);
      // Add the new hover handler for regions
      once(
        'vlsuite-layout-builder-region-hover',
        '.layout-builder__region',
        context,
      ).forEach(layoutBuilderRegionHover);
    },
  };

  /**
   * Layout builder edit live preview.
   */
  function layoutBuilderEditLivePreview(editorFor) {
    const editorElement = document.querySelector(
      `#${editorFor.getAttribute('data-editor-for')}`,
    );
    const originalValue = editorElement.getAttribute(
      'data-editor-value-original',
    );
    const fieldClass = `field--name-${editorElement
      .getAttribute('name')
      .split('settings[block_form][')[1]
      .split(']')[0]
      .replaceAll('_', '-')}`;
    const uuid = editorElement
      .closest('[data-layout-builder-target-highlight-id]')
      .getAttribute('data-layout-builder-target-highlight-id');
    const targetElement = document.querySelector(
      `[data-layout-block-uuid="${uuid}"] .${fieldClass}`,
    );
    if (
      targetElement !== null &&
      editorElement.hasAttribute('data-ckeditor5-id')
    ) {
      $(editorElement).on('formUpdated', function (e) {
        targetElement.innerHTML = Drupal.CKEditor5Instances.get(
          editorElement.getAttribute('data-ckeditor5-id'),
        ).getData();
        $('div#vlsuite-modal').one('dialogclose', function () {
          targetElement.innerHTML = originalValue;
        });
      });
      const smallTag = document.createElement('small');
      smallTag.appendChild(
        document.createTextNode(
          Drupal.t('*Live edit preview active for this field.'),
        ),
      );
      smallTag.classList.add('vlsuite-layout-builder-edit-live-preview-mark');
      editorElement.parentNode.appendChild(smallTag);
      $(window).one(
        'dialog:beforecreate',
        function (e, dialog, $dialogElement, dialogSettings) {
          // Option "dialogClass" deprecated in 1.12 in favor of "classes".
          dialogSettings.dialogClass = 'ui-dialog-fixed';
          dialogSettings.classes = { 'ui-dialog': 'ui-dialog-fixed' };
          dialogSettings.resizable = false;
          dialogSettings.autoResize = false;
          dialogSettings.draggable = true;
          dialogSettings.position = {
            my: 'right top',
            at: 'right-10 bottom',
            of: window,
          };
        },
      );
    }
  }

  /**
   * Layout builder region hover effect.
   * Reduces the parent wrapper's border width when hovering over a region.
   */
  function layoutBuilderRegionHover(region) {
    // Store the original border before any interaction
    const wrapper = region.closest('.vlsuite-layout-bg-wrapper');
    if (wrapper && !wrapper.dataset.originalBorder) {
      // Store the default border from CSS, not the current computed style which might be affected by hover
      wrapper.dataset.originalBorder = 'var(--vlsuite-border)';
    }

    // Add mouseover event
    region.addEventListener('mouseover', function () {
      if (wrapper) {
        // Set thinner border with the hover style
        wrapper.style.border = 'var(--vlsuite-border)';
      }
    });

    // Add mouseout event to restore the original border
    region.addEventListener('mouseout', function () {
      if (wrapper) {
        // First remove any inline style to let CSS take over
        wrapper.style.border = '';

        // Force a reflow to ensure styles are applied
        void wrapper.offsetWidth;

        // If the mouse is still over the wrapper itself, the CSS hover state will apply
        // If not, it will return to the default state
      }
    });
  }

  /**
   * Layout builder live preview.
   */
  function layoutBuilderLivePreview(toggler) {
    toggler.addEventListener('click', function (e) {
      e.preventDefault();
      document
        .querySelector('.layout-builder')
        .classList.toggle('live-preview-active');
    });
  }
})(jQuery, Drupal, once, window);
